/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Frontend - text-file-visualizer.service.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2023 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  Inject,
  Injectable,
} from "@angular/core";
import {HighlightJS} from "ngx-highlightjs";
import {tap} from "rxjs/operators";
import {DefaultSolidifyEnvironment} from "../../core/environments/environment.solidify-defaults";
import {ENVIRONMENT} from "../../core/injection-tokens/environment.injection-token";
import {isTruthyObject} from "../../core/tools/is/is.tool";
import {SsrUtil} from "../../core/utils/ssr.util";
import {FileInput} from "../models/file-info.model";
import {AbstractFileVisualizer} from "./abstract-file-visualizer.service";

@Injectable({
  providedIn: "root",
})
export class TextFileVisualizerService extends AbstractFileVisualizer {
  isFullScreenSupported: boolean = true;
  isZoomSupported: boolean = false;
  type: string = "textPlugin";
  pre: HTMLPreElement = null;

  constructor(private readonly highlight: HighlightJS,
              @Inject(ENVIRONMENT) protected readonly _environment: DefaultSolidifyEnvironment) {
    super(_environment,
      _environment.visualizationTextFileExtensions,
      _environment.visualizationTextFileContentType,
      _environment.visualizationTextFileMimeType,
      _environment.visualizationTextFilePronomId,
      _environment.visualizationTextFileMaxFileSizeInMegabytes);
  }

  isVisualizationOnGoing(fileInfo: FileInput, domElement: Element): boolean {
    return isTruthyObject(this.pre);
  }

  closeVisualizer(fileInfo: FileInput, domElement: Element): void {
    domElement.remove();
    this.pre = null;
  }

  openVisualizer(fileInfo: FileInput, domElement: Element): void {
    this.pre = SsrUtil.window?.document.createElement("pre");
    this.pre.style.display = "block";
    this.pre.style.lineHeight = "1.42857143";
    this.pre.style.fontSize = "13px";
    this.pre.style.margin = "0 0 10px";
    this.pre.style.padding = "9.5px";
    this.pre.style.color = "#333";
    this.pre.style.overflowWrap = "break-word";
    this.pre.style.wordWrap = "break-word";
    this.pre.style.whiteSpace = "break-spaces";
    this.pre.style.backgroundColor = "#f5f5f5";
    this.pre.style.border = "1px solid #ccc";
    this.pre.style.borderRadius = "4px";
    const fr = new FileReader();
    fr.onload = (
      (e) => {
        this.highlight.highlightAuto(fr.result as string, ["xml", "json", "java"]).pipe(
          tap(value => {
            this.pre.innerHTML = value.value;
          }),
        ).subscribe();
        this.highlight.highlightElement(this.pre);
      }
    );
    fr.readAsText(fileInfo.blob, "UTF-8");
    domElement.appendChild(this.pre);
  }

  doAction(fileInfo: FileInput, domElement: Element): void {
  }

  handleZoom(zoomEnable: boolean, fileInfo: FileInput, domElement: Element): void {
  }
}
